const express = require('express');
const http = require('http');
const path = require('path');
const passport = require('passport');
const morgan = require('morgan');
const cookieParser = require('cookie-parser');
const bodyParser = require('body-parser');
const session = require('express-session');
const errorhandler = require('errorhandler');
const flash = require('express-flash-2');
const fileUpload = require('express-fileupload');
const helmet = require('helmet');
const compression = require('compression');
const methodOverride = require('method-override');

var env = process.env.NODE_ENV || 'testing';
const config = require('./config/config')[env];
const lang = 'DE';

var app = express();

app.set('port', config.app.port);
app.set('views', __dirname + '/views');
app.set('view engine', 'pug');

// enable files upload
app.use(fileUpload({
  createParentPath: true,
  limits: { 
    fileSize: 1000000 // 1 MB max. file size
  }
}));

app.use(methodOverride('_method'));
app.use(helmet());
app.use(compression());
app.use(morgan('combined'));
app.use(cookieParser());
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({extended: false}));
app.use(express.static(path.join(__dirname, 'public')));
app.use((req, res, next) => {
  next();
});

app.use(session(
  {
    resave: true,
    saveUninitialized: true,
    secret: config.app.sessionSecret
  }
));
app.use(flash());
app.use(passport.initialize());
app.use(passport.session());

// caching disabled for every route
// NOTE: Works in Firefox and Opera. Does not work in Edge
app.use(function(req, res, next) {
  res.set('Cache-Control', 'no-cache, private, no-store, must-revalidate, max-stale=0, post-check=0, pre-check=0');
  next();
});

require('./routes/account')(app, config, passport, lang);
require('./routes/public')(app, config, lang);

// Handle 404
app.use(function (req, res) {
  res.status(404).render(lang+'/404')
})

// Handle 500 - any server error
app.use(function (err, req, res, next) {
  console.error(err.stack)
  res.status(500).render(lang+'/500', {
    error: err
  })
})

app.listen(app.get('port'), function () {
  console.log('Express server listening on port ' + app.get('port'));
});